from Components.ActionMap import ActionMap
from Screens.Screen import Screen
from Components.ConfigList import ConfigListScreen
from Components.config import config, ConfigOnOff, ConfigSubsection, getConfigListEntry, ConfigSubList, ConfigText, ConfigInteger, configfile
from Components.ServiceEventTracker import ServiceEventTracker
from Components.Label import Label
from Components.Pixmap import Pixmap
from enigma import ePoint, getDesktop, iPlayableService, iServiceInformation
from Plugins.Plugin import PluginDescriptor
from os import listdir as os_listdir

config.plugins.merlinwidgets = ConfigSubsection()
config.plugins.merlinwidgets.widgets = ConfigSubList()
config.plugins.merlinwidgets.count = ConfigInteger(0)

widgetHandler = None

def initWidgetConfig():
	s = ConfigSubsection()
	s.name = ConfigText(default = "")
	s.active = ConfigOnOff(default=False)
	s.permanent = ConfigOnOff(default=False)
	s.hideonfta = ConfigOnOff(default=False)
	
	config.plugins.merlinwidgets.widgets.append(s)
	return s
	
def initConfig():
	count = config.plugins.merlinwidgets.count.value
	if count != 0:
		i = 0
		while i < count:
			initWidgetConfig()
			i += 1
			
initConfig()

def autostart(reason, **kwargs):
	if reason == 0:
		from skin import loadSkin, loadSingleSkinData, dom_skins

		for skinXml in os_listdir("/etc/enigma2/MerlinWidgets/"):											
			skinFile = "/etc/enigma2/MerlinWidgets/%s" %(skinXml)
			loadSkin(skinFile, "")
			path, dom_skin = dom_skins[-1:][0]
			loadSingleSkinData(getDesktop(0), dom_skin, path)
		

def Plugins(**kwargs):
	return [PluginDescriptor(where = PluginDescriptor.WHERE_INFOBAR, fnc = startWidget),PluginDescriptor(where = PluginDescriptor.WHERE_PLUGINMENU, name="MerlinWidgets Setup", description="Configure MerlinWidgets", fnc= startConfig, icon="MerlinWidgets.png"), PluginDescriptor(where = PluginDescriptor.WHERE_SESSIONSTART, fnc = autostart)]

def startConfig(session):
	session.open(MerlinWidgetsConfig)
	
class MerlinWidgetsConfig(Screen, ConfigListScreen):
	skin = """
			<screen position="center,center" size="800,650" title="MerlinWidgets - Config">
				<widget name="config" position="0,0" size="800,600" scrollbarMode="showOnDemand" />
				<widget name="ButtonGreentext" position="20,610" size="350,40" valign="center" halign="left" zPosition="10" font="Regular;18" transparent="1" />
				<widget name="ButtonGreen" pixmap="skin_default/buttons/button_green.png" position="0,620" zPosition="10" size="16,16" transparent="1" alphatest="on" />
			</screen>
		"""
        
	def __init__(self,session):
		self.session = session
		Screen.__init__(self, session)
		
		self["actions"] = ActionMap(["OkCancelActions","ColorActions"],
			{
				"ok":	self.showWidgetConfig,
				"cancel": self.keyCancel,
				"green": self.saveSelection,
			}, -1)
	
		self["ButtonGreen"] = Pixmap()
		self["ButtonGreentext"] = Label(_("Save settings"))
		
		self.list = []
		
		ConfigListScreen.__init__(self, self.list)
		
		self.getWidgets()
		
	def saveSelection(self):
		for x in self["config"].list:
			if x[1].isChanged() and x[1].value:
				widgetName = x[2]
				self.loadSingleSkin(widgetName)
				if widgetHandler.widgetDict.get(widgetName, None) is None:
					print "[MerlinWidgets] - widget is not yet initialised"
					widgetHandler.initSingleWidget(widgetName)
			x[1].save()
                        
		self.close()

	def showWidgetConfig(self):
		index = self["config"].getCurrentIndex()
		self.session.open(WidgetConfig, index)
		
	def getWidgets(self):
		self.widgetList = []
		currentList = []

		count = config.plugins.merlinwidgets.count.value
		if count != 0:
			for i in range(0, count):
				currentList.append(config.plugins.merlinwidgets.widgets[i].name.value)
			
		for f in os_listdir("/etc/enigma2/MerlinWidgets/"):
			widgetName = f[:-4]
			self.widgetList.append(widgetName)

			# add new widgets to config
			if widgetName not in currentList:
				newEntry = initWidgetConfig()
				newEntry.name.value = widgetName
				newEntry.save()
				config.plugins.merlinwidgets.count.value += 1
				config.plugins.merlinwidgets.count.save()
				config.plugins.merlinwidgets.save()
				configfile.save()				
			else:
				# widget is already in config. Remove from list.
				currentList.remove(widgetName)
			
		# now remove all widgets that are no longer available (and avoid crash when user activates it)
		if len(currentList):
			for removedItem in currentList:
				count = config.plugins.merlinwidgets.count.value
				for i in range(0, count):
					if config.plugins.merlinwidgets.widgets[i].name.value == removedItem:
						config.plugins.merlinwidgets.count.value -= 1
						del config.plugins.merlinwidgets.widgets[i]
						config.plugins.merlinwidgets.count.save()
						config.plugins.merlinwidgets.save()
						configfile.save()
						break
							
		self.createConfig()
	
	def loadSingleSkin(self, file):
		print "[MerlinWidgets] - widget activated - loading skin"
		from skin import loadSkin, loadSingleSkinData, dom_skins

		skinFile = "/etc/enigma2/MerlinWidgets/%s.xml" %(file)
		for skin in dom_skins:
			if skin[0] == "/etc/enigma2/MerlinWidgets/" and skin[1].find('screen').get('name', "") == file:
				print "[MerlinWidgets] - skin already loaded"
				break
		else:
			loadSkin(skinFile, "")
			path, dom_skin = dom_skins[-1:][0]
			loadSingleSkinData(getDesktop(0), dom_skin, path)
				
	def createConfig(self, configElement=""):
		self.list = []
		count = config.plugins.merlinwidgets.count.value
		if count != 0:
			for i in range(0, count):
				self.list.append(getConfigListEntry(_("Activate %s Widget" %(config.plugins.merlinwidgets.widgets[i].name.value)), config.plugins.merlinwidgets.widgets[i].active, config.plugins.merlinwidgets.widgets[i].name.value))
		self["config"].setList(self.list)

class WidgetConfig(Screen, ConfigListScreen):
	skin = """
			<screen position="center,center" size="800,650" title="MerlinWidgets - WidgetConfig">
				<widget name="config" position="0,0" size="800,600" scrollbarMode="showOnDemand" />
				<widget name="ButtonGreentext" position="20,610" size="350,40" valign="center" halign="left" zPosition="10" font="Regular;18" transparent="1" />
				<widget name="ButtonGreen" pixmap="skin_default/buttons/button_green.png" position="0,620" zPosition="10" size="16,16" transparent="1" alphatest="on" />
			</screen>
		"""	
	def __init__(self, session, index):
		self.session = session
		self.index = index
		Screen.__init__(self, session)
		
		self.setTitle(_("MerlinWidgets - WidgetConfig: %s" %(config.plugins.merlinwidgets.widgets[index].name.value)))
		
		self["OkCancelActions"] = ActionMap(["OkCancelActions","ColorActions"],
			{
				"cancel": self.keyCancel,
				"green": self.keySave,
			}, -1)

		self["ButtonGreen"] = Pixmap()
		self["ButtonGreentext"] = Label(_("Save settings"))
	
		self.list = []
		
		self.list.append(getConfigListEntry(_("Do not hide with InfoBar"), config.plugins.merlinwidgets.widgets[index].permanent))
		self.list.append(getConfigListEntry(_("Do not show on FTA channels"), config.plugins.merlinwidgets.widgets[index].hideonfta))
		
		ConfigListScreen.__init__(self, self.list)
		
def startWidget(session):
	global widgetHandler 
	if widgetHandler is None:
		widgetHandler = WidgetHandler(session)

class WidgetHandler:
	def __init__(self,session):
		from Screens.InfoBar import InfoBar
		import Screens.InfoBar
		
		self.infoBarInstance = InfoBar.instance
		self.infoBarInstance.onShow.append(self.showWidget)
		self.infoBarInstance.onHide.append(self.hideWidget)
		
		# listen on evPlay event to hide on FTA?
		
		self.session = session
		self.isFTA = True
		self.onClose = []
		
		self.__event_tracker = ServiceEventTracker(screen=self, eventmap=
			{
				iPlayableService.evUpdatedInfo:	self.serviceInfoUpdated,
			})
		
		self.widgetDict = {}
		count = config.plugins.merlinwidgets.count.value
		if count != 0:
			for i in range(0, count):
				widgetName = config.plugins.merlinwidgets.widgets[i].name.value
				if config.plugins.merlinwidgets.widgets[i].active.value:
					print "[MerlinWidgets] - %s activated. Init." %(widgetName)
					self.widgetDict[widgetName] = None
					self.widgetDict[widgetName] = self.session.instantiateDialog(GenericScreen, skinName=widgetName)
					self.widgetDict[widgetName].neverAnimate()
				else:
					print "[MerlinWidgets] - %s not activated. No need to init" %(widgetName)
	
	def serviceInfoUpdated(self):
		service = self.session.nav.getCurrentService()
		info = service and service.info()
		self.isFTA = True
		if info:
			caids = info.getInfoObject(iServiceInformation.sCAIDs)
			if caids:
				self.isFTA = False

		if self.infoBarInstance.shown:
			self.showOrHide()
	
	def showOrHide(self):
		count = config.plugins.merlinwidgets.count.value
		if count != 0:
			for i in range(0, count):
				# for each activated widget
				if config.plugins.merlinwidgets.widgets[i].active.value:
					currentWidget = config.plugins.merlinwidgets.widgets[i].name.value
					# if don't show widget on FTA and channel is FTA and widget is shown and infobar is still shown
					if config.plugins.merlinwidgets.widgets[i].hideonfta.value and self.isFTA and self.widgetDict[currentWidget].shown:
						# hide widget
						self.widgetDict[currentWidget].hide()
					# if channel is not FTA and widget is not shown and infobar is still shown	
					elif not self.isFTA and not self.widgetDict[currentWidget].shown and self.infoBarInstance.shown:
						# show widget
						self.widgetDict[currentWidget].show()
				
	def initSingleWidget(self, widgetName):
		print "[MerlinWidgets] - init single widget"
		self.widgetDict[widgetName] = None
		self.widgetDict[widgetName] = self.session.instantiateDialog(GenericScreen, skinName=widgetName)
		self.widgetDict[widgetName].neverAnimate()
				
	def showWidget(self):
		count = config.plugins.merlinwidgets.count.value
		if count != 0:
			for i in range(0, count):
				if config.plugins.merlinwidgets.widgets[i].active.value:
					if not config.plugins.merlinwidgets.widgets[i].hideonfta.value or (config.plugins.merlinwidgets.widgets[i].hideonfta.value and not self.isFTA):
						self.widgetDict[config.plugins.merlinwidgets.widgets[i].name.value].show()
		
	def hideWidget(self):
		count = config.plugins.merlinwidgets.count.value
		if count != 0:
			for i in range(0, count):
				if config.plugins.merlinwidgets.widgets[i].active.value:
					if not config.plugins.merlinwidgets.widgets[i].permanent.value:
						self.widgetDict[config.plugins.merlinwidgets.widgets[i].name.value].hide()	



class GenericScreen(Screen):
	def __init__(self, session, **kwargs):
		Screen.__init__(self, session)
		self.skinName = kwargs["skinName"]